@@ -31,7 +31,7 @@
 
 #include <vector>
 #include <list>
-//#include <algorithm>
+#include <algorithm>
 
 // #include "linbox/field/multimod-field.h"
 #include "linbox/matrix/dense-matrix.h"
@@ -61,14 +61,14 @@ namespace LinBox
         std::list<Elt>& smithStep(std::list<Elt>& L, Elt& d, Matrix& A, const LocalPID& R)
         {
             if ( A.rowdim() == 0 || A.coldim() == 0 ) return L;
-        	
+
             Elt g; R.assign(g, R.zero);
             typename Matrix::RowIterator p;
             typename Matrix::Row::iterator q, r;
             for ( p = A.rowBegin(); p != A.rowEnd(); ++p) {
                 for (q = p->begin(); q != p->end(); ++q) {
                     R.gcdin(g, *q);
-                    
+
                     if ( R.isUnit(g) ) {
                         //R.divin(g, g); break;
                         R.assign(g, R.one); break;
@@ -84,46 +84,45 @@ namespace LinBox
             }
 
             if ( p != A.rowEnd() ) {
-            	
+
                 // g is a unit and,
                 // because this is a local ring, value at which this first happened
                 // also is a unit.
                 if ( p != A.rowBegin() )
-                    swap_ranges(A.rowBegin()->begin(), A.rowBegin()->end(), p->begin());
-                
+                    std::swap_ranges(A.rowBegin()->begin(), A.rowBegin()->end(), p->begin());
+
                 if ( q != p->begin() )
-                    swap_ranges(A.colBegin()->begin(), A.colBegin()->end(),
+                    std::swap_ranges(A.colBegin()->begin(), A.colBegin()->end(),
                             (A.colBegin() +(int) (q - p->begin()))->begin());
 
                 // eliminate step - crude and for dense only - fix later
                 // Want to use a block method or "left looking" elimination.
                 Elt f; R.inv(f, *(A.rowBegin()->begin() ) );
                 R.negin(f);
-                
+
                 // normalize first row to -1, ...
                 for ( q = A.rowBegin()->begin() /*+ 1*/; q != A.rowBegin()->end(); ++q)
                     R.mulin(*q, f);
-                
+
                 //
                 // eliminate in subsequent rows
-                size_t i = 0, j = 0;
                 for (p = A.rowBegin() + 1; p != A.rowEnd(); ++p) {
-                	i++;
                     for (q = p->begin() + 1, r = A.rowBegin()->begin() + 1, f = *(p -> begin()); q != p->end(); ++q, ++r) {
-                    	j++;
-                    	
                         R.axpyin( *q, f, *r );
                     }
                 }
 
-                BlasMatrix<LocalPID> Ap(A, 1, 1, A.rowdim() - 1, A.coldim() - 1);
+				// submatrix!
+                BlasSubmatrix<BlasMatrix<LocalPID> > Ap(A, 1, 1, A.rowdim() - 1, A.coldim() - 1);
                 L.push_back(d);
                 return smithStep(L, d, Ap, R);
             } else {
                 typename Matrix::Iterator p_it;
                 for (p_it = A.Begin(); p_it != A.End(); ++p_it) {
-                    R.divin(*p_it, g);
+                   R.divin(*p_it, g);
                 }
+				typename LocalPID::Element x; R.neg(x, g);
+				R.gcdin(g,x);
                 return smithStep(L, R.mulin(d, g), A, R);
             }
         }
@@ -133,11 +132,10 @@ namespace LinBox
 #include "linbox/algorithms/smith-form-local2.inl"
 #endif // __LINBOX_smith_form_local_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
-// tab-width: 8
+// tab-width: 4
 // indent-tabs-mode: nil
-// c-basic-offset: 8
+// c-basic-offset: 4
 // End:
-
+// vim:sts=4:sw=4:ts=4:et:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
